#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_aitx
----------------------------------

Tests for `aitx` module.
"""

import unittest

from aitx import aitx


class TestAitx(unittest.TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
test_imagebox
----------------------------------

Tests for `imagebox` module.
"""

import unittest
import numpy as np

from aitx.imagebox import ImageBox


def _create_image(grayscale=True, shape=(10, 10)):
    if grayscale:
        size = shape[0] * shape[1]
        gray = np.random.randint(150, 256, shape).astype('float32')
        gray[2:8, 2:8] = 128
        gray[4:6, 4:6] = 0
        return gray
    else:
        shape = (*shape[:2], 3)
        size = shape[0] * shape[1] * 3
        rgb = np.random.randint(150, 256, shape).astype('float32')
        rgb[2:8, 2:8, :] = 128.0
        rgb[4:6, 4:6, :] = 0.0
        return rgb

def create_images(size=10, **kwarg):
    raw_images = [_create_image(**kwarg) for _ in range(size)]
    shape = raw_images[0].shape
    new_shape = (1, *shape)
    images = np.concatenate([i.reshape(new_shape) for i in raw_images])
    return images


class TestImagebox(unittest.TestCase):

    def setUp(self):
        self.size = 10
        self.shape = (10, 10)
        self.gray = create_images(self.size)
        self.rgb = create_images(self.size, grayscale=False)
        self.normal_gray = (self.gray / 255).reshape(10, 10, 10, 1)
        self.normal_rgb = self.rgb / 255
        self.labels = np.array([0, 1]*int(self.size/2))
        self.label_dict={0: False, 1: True}

    def test_init(self):
        # gray
        ib = ImageBox(self.gray, self.labels, self.label_dict)
        self.assertTrue(np.all(ib.grayscale==self.gray))
        self.assertEqual(ib.rgb, None)
        self.assertEqual(ib.image4D.shape, (self.size, *self.shape, 1))
        self.assertTrue(np.all(ib.normalize==self.normal_gray))

        # rgb
        ib = ImageBox(self.rgb, self.labels, self.label_dict)
        self.assertEqual(ib.grayscale, None)
        self.assertTrue(np.all(ib.rgb==self.rgb))
        self.assertEqual(ib.image4D.shape, (self.size, *self.shape, 3))
        self.assertTrue(np.all(ib.normalize==self.normal_rgb))

        # normal gray
        ib = ImageBox(self.normal_gray, self.labels, self.label_dict)
        self.assertTrue(np.all(ib.grayscale==self.normal_gray))
        self.assertEqual(ib.rgb, None)
        self.assertEqual(ib.image4D.shape, (self.size, *self.shape, 1))
        self.assertTrue(np.all(ib.normalize==self.normal_gray))

        # normal rgb
        ib = ImageBox(self.normal_rgb, self.labels, self.label_dict)
        self.assertEqual(ib.grayscale, None)
        self.assertTrue(np.all(ib.rgb==self.normal_rgb))
        self.assertEqual(ib.image4D.shape, (self.size, *self.shape, 3))
        self.assertTrue(np.all(ib.normalize==self.normal_rgb))

    def tearDown(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
from collections import defaultdict
import matplotlib.patches as patches

class History:

    def __init__(self):
        self._history_log = []
        self.human_accuracy = None

    def update(self, history):
        self._history_log.append(history)

    def set_human_accuracy(self, score):
        self.human_accuracy = score

    def get_full_history(self):
        full_history = defaultdict(list)
        for history in self._history_log:
            for k, v in history.history.items():
                full_history[k].extend(v)

        return full_history

    def plot(self, train='acc', validation=None):
        if validation is None:
            validation = 'val_' + train

        epoch_left = 0
        for i, history_obj in enumerate(self._history_log):
            train_history = history_obj.history
            epochs = len(train_history[train])
            epoch_right = epoch_left + epochs
            if i % 2 == 1:
                axes = plt.gca()
                axes.add_patch(patches.Rectangle(
                                (epoch_left+0.5, 0),   # (x,y)
                                epochs,         # width
                                1.5,          # height
                                color='#f6f6f6'
                            ))
            epoch_left = epoch_right

        full_history = self.get_full_history()
        full_epochs = len(full_history[train])
        if self.human_accuracy is not None:
            plt.plot([0, full_epochs], [self.human_accuracy, self.human_accuracy], linestyle='--', color='gray', label='human')
        plt.plot(range(1, full_epochs+1), full_history[train], label='train')
        plt.plot(range(1, full_epochs+1), full_history[validation], label='validation')

        plt.title('Train history')
        plt.ylabel(train)
        plt.xlabel('Epoch')
        plt.xticks(range(0, full_epochs+1))
        plt.legend(loc='best')
        axes.set_xlim([0.5, full_epochs+0.5])
        axes.set_ylim([0, 1.1])
        plt.show()

    def show(self, train='acc', validation=None):
        self.plot(train, validation)

    def diagnosis(self, human_accuracy=None):
        if human_accuracy is not None:
            self.human_accuracy = human_accuracy
        if self.human_accuracy is None:
            print("Need human accuracy")
        full_history = self.get_full_history()
        hum_error = 1 - self.human_accuracy
        acc_error = 1 - full_history['acc'][-1]
        val_error = 1 - full_history['val_acc'][-1]
        template = "Human level error:\t{:.3f}\nTraining set error:\t{:.3f}\nDev/Test set error:\t{:.3f}"
        print(template.format(hum_error, acc_error, val_error))
        bias = max(0, acc_error - hum_error)
        variance = val_error - acc_error
        print('Avoidable bias:\t{:.3f}\nVariance:\t{:.3f}'.format(bias, variance))
        if bias >= variance:
            print('Recommend train on a larger model or more time.')
        else:
            print('{:->30}'.format(''))
            print('Recommend more regulation or dropout.')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt

def lightbox(images, titles=None, labels=None, column_num=5, **kwargs):
    """
    Show the images labels in a table

    Parameters
    ----------
    images: array-like or PIL images
            images to show

    titles: list of str
            the titles of the images

    labels: list of str
            the labels of the images

    column_num: int
                the number of columns, the default is 5

    Return
    ------
    None


    Other parameters
    ----------------
    **kwargs: the parameters for matplotlib.pyplot.figure()

    ### Output
    <matplotlib object>

    """
    def ceildiv(a, b):
        return -(-a // b)

    size = len(images)

    if titles is None:
        titles = [''] * size

    if labels is None:
        labels = [''] * size

    rownumber = ceildiv(len(images), column_num)

    fig = plt.figure(**kwargs)
    fig.set_size_inches(12, 2.8 * rownumber)

    for n, (img, title, label) in enumerate(zip(images, titles, labels)):
        ax = plt.subplot(rownumber, 5, n+1)
        cmap = None
        ax.imshow(img, cmap='gray')
        ax.set_title(label, fontsize=12)
        ax.set_xticks([])
        ax.set_yticks([])
        ax.set_xlabel(title)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
import sys
import os

with open('README.md') as readme_file:
    readme = readme_file.read()

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist bdist_wheel upload")
    sys.exit()

requirements = [
    # TODO: put package requirements here
]

setup(
    name='aitx',
    version='0.4.4',
    description="The toolbox for general ML projects",
    long_description=readme,
    author="Chia-Jung, Yang",
    author_email='jeroyang@gmail.com',
    url='https://github.com/jeroyang/aitx',
    packages=[
        'aitx',
    ],
    package_dir={'aitx':
                 'aitx'},
    include_package_data=True,
    install_requires=requirements,
    license="MIT",
    zip_safe=False,
    keywords='aitx',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
)
